from random import randint

from ..interfaces import FightingEntity


class Player(FightingEntity):
    name = "player"
    maxhealth: int = 20
    strength: int = 5
    intelligence: int = 1
    charisma: int = 1
    dexterity: int = 1
    constitution: int = 1
    level: int = 1
    current_xp: int = 0
    max_xp: int = 10

    def move(self, y: int, x: int) -> None:
        """
        When the player moves, move the camera of the map.
        """
        super().move(y, x)
        self.map.currenty = y
        self.map.currentx = x

    def level_up(self) -> None:
        """
        Add levels to the player as much as it is possible.
        """
        while self.current_xp > self.max_xp:
            self.level += 1
            self.current_xp -= self.max_xp
            self.max_xp = self.level * 10
            self.health = self.maxhealth
            # TODO Remove it, that's only fun
            self.map.spawn_random_entities(randint(self.level, self.level * 5))

    def add_xp(self, xp: int) -> None:
        """
        Add some experience to the player.
        If the required amount is reached, level up.
        """
        self.current_xp += xp
        self.level_up()

    def check_move(self, y: int, x: int, move_if_possible: bool = False) \
            -> bool:
        """
        If the player tries to move but a fighting entity is there,
        the player fights this entity.
        It rewards some XP if it is dead.
        """
        # Don't move if we are dead
        if self.dead:
            return False
        for entity in self.map.entities:
            if entity.y == y and entity.x == x and \
                    isinstance(entity, FightingEntity):
                self.hit(entity)
                if entity.dead:
                    self.add_xp(randint(3, 7))
                return True
        return super().check_move(y, x, move_if_possible)
